package packed

import (
	"encoding/json"
	"errors"
	"io/ioutil"
	"net/http"
	"net/http/cookiejar"
	"net/url"
	"regexp"
	"strings"
	"time"

	"ppt_server/app/model"

	"github.com/gogf/gf/frame/g"
	"github.com/gogf/gf/util/gconv"
)

type Notfiy struct {
	token    string
	name     string
	password string
	url      string
	c        *http.Client
}

func NewNotfiy() *Notfiy {
	jar, _ := cookiejar.New(nil)
	return &Notfiy{
		url:      g.Cfg().GetString("notify.url"),
		name:     g.Cfg().GetString("notify.name"),
		password: g.Cfg().GetString("notify.password"),
		c: &http.Client{
			Jar: jar,
			CheckRedirect: func(req *http.Request, via []*http.Request) error {
				return http.ErrUseLastResponse
			},
		},
	}
}

func (n *Notfiy) Post(url string, data g.Map) error {

	res, err := n.request("POST", url, data)
	if err != nil {
		return err
	}
	var login model.NotifyResponse
	if err := json.Unmarshal(res, &login); err != nil {
		return err
	}

	if login.Code != 200 {
		return errors.New(login.Msg)
	}
	return nil
}

func (n *Notfiy) Get(url string, data g.Map) ([]byte, error) {
	return n.request("GET", url, data)
}

func (n *Notfiy) request(method, r_url string, data g.Map) ([]byte, error) {

	vv := url.Values{}
	for k, v := range data {
		vv.Add(k, gconv.String(v))
	}
	vv.Encode()

	reader := strings.NewReader(vv.Encode())
	req, err := http.NewRequest(method, r_url, reader)
	if err != nil {
		return nil, err
	}
	req.Header.Set("Content-Type", "application/x-www-form-urlencoded")

	res, err := n.c.Do(req)
	if err != nil {
		return nil, err
	}
	defer res.Body.Close()

	return ioutil.ReadAll(res.Body)
}

func (n *Notfiy) setToken() error {
	if err := n.Post(n.url+"/web/login", g.Map{
		"name":     n.name,
		"password": n.password,
	}); err != nil {
		return err
	}

	// 获取Token
	tokenTypes, err := n.Get(n.url+"/web/admin", nil)
	if err != nil {
		return err
	}
	re := regexp.MustCompile(`<meta name="csrf-token" content="([^"]+)">`)
	token := re.FindAllSubmatch(tokenTypes, -1)
	n.token = string(token[0][1])
	return nil
}

// 上传失败通知
func (n *Notfiy) Notify(path string, data g.Map) {

	var err error
	if err = n.setToken(); err != nil {
		g.Log().Async().Error(err)
		return
	}
	data["_token"] = n.token
	if err = n.Post(n.url+path, data); err != nil {
		g.Log().Async().Errorf("notify fail, err : %s\n", err)
	}
	g.Log().Async().Infof("Notify Success!")
	return
}

func FailGmap(r *model.FileUploadRequest, object *ObtainFile) g.Map {
	return g.Map{
		"code":     400,
		"uuid":     r.Uuid,
		"room_num": r.RoomNum,
		"path":     object.DataPath,
	}
}

func SuccessGmap(r *model.FileUploadRequest, object *ObtainFile, room_id int) g.Map {
	data := g.Map{
		"room_id":        room_id,
		"is_courseware":  "1",
		"private":        "1",
		"nickname":       r.Nickanem,
		"name":           r.Name,
		"time":           time.Now().Format("2006-01-02 15:04:05"),
		"uuid":           r.Uuid,
		"file_name_hash": object.NameHash,
		"hash":           object.ContentHash,
		"doc_type":       object.DocType,
		"type":           object.MineType,
		"path":           object.DataPath,
		"files_size":     object.FileSize,
		"link":           object.Link,
		"page_count":     object.PageCount,
	}

	switch r.Type {
	case 1:
		data["is_material"] = 1
	case 2:
		data["is_courseware"] = 1
	case 3:
		data["is_titlebook"] = 1
	}

	return data
}
